/* $Id: dumpgroups.c,v 1.1.2.1 2006/07/02 19:58:04 mc Exp $
**
** Test program for for users and access control lists.
**
** M.C. Widerkrantz, mc at hack.org
**
*/

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sysexits.h>
#include <string.h>
#include <db.h>

/* FIXME: Use pathdb in struct innconf to create these. */

#define GROUPSDB "/usr/local/news/db/newsgroups.db"
#define USERSDB "/usr/local/news/db/newsusers.db"
#define ACLDBPREFIX "/usr/local/news/db/acl/"


#define GROUPMAX 6
struct newsgroup
{
    char creator[128];
    char moderator[128];
    int expire;
    int flags;
    char title[182];
    char description[128];
};


static void hexdump(u_int8_t *dp, u_int16_t len)
{
  u_int8_t *p, *q, *max;
  int done;

  p = dp;
  max = &dp[len];
  
  for (done = 0; !done; )
  {
    /* Print byte offset from dp. */
    printf("%07x ", p - dp);

    for (q = p; q != max && q != (p + 16); q ++)
    {
      printf("%02x ", *q);
    }

    (void)putchar('\n');
    
    if (q == max)
    {
      done = 1;
    }
    else
    {
      /* Next row */
      p = q;
    }
  } /* Outer for */
}

/*
 * Separate data into an array of strings, stored in token.
 */

int
tokenize(DBT data, char *token[], int max)
{
    char *s, *t, **tp;
    int p, nr = 0;

    /* Split at NUL */

    s = data.data;
    t = s;

    for (p = 0, tp = token; p <= data.size; p ++)
    {
        if (s[p] == '\0')
        {
            *tp = t;
            tp ++;
            if (nr == max)
                break;
            else
                nr ++;
            t = &s[p + 1];
        }
    }

    return nr;
}

int
main(void)
{
    DB *db;
    DBT key, data;
    DBC *dbc;
    int rc;
    int fields;
    int i;
    char *fname = GROUPSDB;
    char *token[GROUPMAX];
    
   /* Create the database handle... */

    if ((rc = db_create(&db, NULL, 0)) != 0)
    {
	fprintf(stderr, "db_create:  %s\n", db_strerror(rc));
        exit(1);
    }

    /* Open existing database. */
    if (0 != (rc = (db->open(db, NULL, fname, NULL, DB_HASH, 0,
                             0644))))
    {
	db->err(db, rc, "%s", fname);
        exit(1);
    }

    memset(&key, 0, sizeof key);
    memset(&data, 0, sizeof data);

    if (0 != (rc = db->cursor(db, NULL, &dbc, 0)))
    {
	db->err(db, rc, "%s", fname);        
        exit(1);
    }

    while (rc != DB_NOTFOUND)
    {
        if (0 != (rc = dbc->c_get(dbc, &key, &data, DB_NEXT)))
        {
            break;
        }

        printf("Key: %s\nData:\n", key);
        hexdump(data.data, data.size);

        fields = tokenize(data, token, GROUPMAX);

        if (GROUPMAX != fields)
        {
            printf("Number of fields (%d) more than maximum allowed (%d)\n",
                   fields, GROUPMAX);
        }

        printf("Fields:\n");
        for (i = 0; i < fields; i ++)
        {
            printf("%d: %s\n", i, token[i]);
        }        
    }

    exit(0);
}
